// REXX DLL for simple process information. (c)1999 by VikingX
// Contained:
// ProcList - Get list of processes in REXX list STEM.0 containes
//            count of processes. Usage: call ProcList [StemVariable].
//            See REXX examlpe.
// Used undocumented function DosQuerySysState

Library RxProcss;

Uses
  Use32, Os2Def, Os2Rexx, Strings, Procman;

{$CDecl+,OrgName+,I-,S-,Delphi+,X+}

{$D RXPROCSS - Simple REXX Process Informer 1.0.0}    // DLL description

{$LINKER

  DATA MULTIPLE NONSHARED

  EXPORTS
    PROCLIST     = ProcList
    PRCLOADFUNCS = PrcLoadFuncs

}

Const
  Null: PChar = '???';
  BufSize: ULong = $10000;
  MaxNameSz = $1000;
  FunctionName : pChar  =  'ProcList';
var
  StemName,ProcessName : PChar;

function SetStemNode(var P: ShvBlock; Value: pchar; const n: ULong): ApiRet;
{Added one item to REXX stem variable. First word - PID, second - Name}
var
    s         : string;
    ps        : pchar;
begin
   Str(n,s);                    // Get number of stem item
   s := '.' + s + #0;           // .NNN#0 dot ahead and null for terminate
   ps := PChar(@s[1]);          // ps now point to null-terminated string
   with P do begin
        shvnext := Nil;
        GetMem(shvname.strptr, strlen(StemName)+strlen(ps)); // memory
                                            // for summary StemName&ItemNum
        strcopy(shvname.strptr,StemName);   // copy stemname
        strcat(shvname.strptr,ps);          // and add number with dot ahead
        shvname.strlength := strlen(shvname.strptr); // save summary length (for RXString type)
        shvnamelen := shvname.strlength;
        shvvalue.strptr := Value;
        shvvalue.strlength := strlen(Value);
        shvvaluelen := shvvalue.strlength;
        shvcode := rxshv_Set;
        shvret := 0;

        SetStemNode := RexxVariablePool(P); // ok. now set variable

        FreeMem(shvname.strptr, strlen(StemName)+strlen(ps)); // memory free
   end;
end;


function GetModule(Buf: PQTopLevel; HNDMod: ULong): PChar;
{Get process name. Inner function}
var
  P: PQModule;
begin
  P:= Buf^.ModData;
  while Assigned(P)  // not NIL
  do begin
     if P^.HNDMod = HNDMod
     then begin
          GetModule := P^.Name;
          exit;
          end;
     P:= P^.Next;
     end;

  GetModule := Null;
end;


procedure DumpProcesses(PB: PShvBlock; Buf: PQTopLevel);
var
  PrcNam, PrcID: PChar;
  P: PQProcess;
  T: PQThread;
  I: ULong;
  tmp: string;
begin
  I:=0;
  P:= Buf^.ProcData;
  while P^.RecType <> 3
  do begin
     inc(I);
     PrcNam := GetModule(Buf, P^.HNDMod);
     Str(P^.PID,tmp);
     tmp := tmp + ' ' + chr(0);
     GetMem(PrcID, strlen(PChar(@tmp[1]))+strlen(PrcNam)+1); // note: +1!

     strcopy(PrcID,PChar(@tmp[1]));
     strcat(PrcID,PrcNam); // Now PrcID contain PID and process name

     SetStemNode(PB^,PrcID,i);

     FreeMem(PrcID, strlen(PChar(@tmp[1]))+strlen(PrcNam)+1);

     T:= P^.Threads;
     Inc(T, P^.ThreadCnt);
     P:= PQProcess(T);
     end;
  str(i, tmp); tmp:=tmp+chr(0);
  PrcNam := PChar(@tmp[1]);
  SetStemNode(PB^,PrcNam,0);
end;

Function PrcLoadFuncs( FuncName  : PChar;
                       ArgC      : ULong;
                       Args      : pRxString;
                       QueueName : pChar;
                       Var Ret   : RxString ) : ULong; export;
begin
  Ret.strLength := 0;
  If ArgC > 0
  then                        //Do not allow parameters
    PrcLoadFuncs := 40
  else begin
       RexxRegisterFunctionDLL( FunctionName, 'RXPROCSS', FunctionName );
       PrcLoadFuncs := 0;
       end;
end;


Function ProcList( Name      : PChar;
                 ArgC      : ULong;
                 Args      : pRxString;
                 QueueName : pChar;
                 Var Ret   : RxString ) : ULong; export;

var R         : ShvBlock;
    num       : ULONG;
    Buf       : PQTopLevel;
    RC        : ApiRet;


begin
  StemName := Args^.strptr;
//  if ArgC <> 1 then Exit(3);
  GetMem(Buf, BufSize);
  if Assigned(Buf)
  then begin
       FillChar(Buf^, BufSize, 0);
       RC:= DosQuerySysState($1F, 0, 0, 0, Buf^, BufSize);
       if RC = 0
       then begin
            DumpProcesses(@R,Buf);
            end
       else WriteLn('DosQuerySysState = ', RC);
       FreeMem(Buf, BufSize);
       end
  else WriteLn('ProcList: No Memory');

  ProcList := 0
end;

initialization
end.

